/*======================================================================*
 *  Module .............libbtwam
 *  File ...............Haptics.h
 *  Author .............Sam Clanton
 *  Creation Date ......Q3 2004
 *  Addt'l Authors......Traveler Hauptman
 *                                                                      *
 *  ******************************************************************  *
 *                                                                      *
 *  NOTES:
 *
 *  REVISION HISTORY:
 *    Dec 17, 2004: TH
 *      Splitting libbt into libbtsystem & libbtwam. Adjust file names
 *      to support this. btwam -> btwamctl, btdriver -> btwam
 *                                                                      *
 *======================================================================*/

#ifndef __HAPTICS_H__
#define __HAPTICS_H__


#include "Kinematics.h"
#include "Dynamics.h"
#include "btwam.h"


#define MAX_FORCE_EFFECTORS 16
#define MAX_HAPTIC_OBJECTS 256

typedef struct {
  
  wam_struct *wamDriverData; //temporarily a pointer
  WAMLoc wamTipLoc;
  WAMKinematics wamKin;
  WAMDynamics wamDyn;


} WAMdat;


typedef int (*ForceEffector)(WAMdat *wd, WAMLoc *resultForce, int id);

typedef int (*ObjectInteract)(WAMLoc *wamTipLoc, WAMLoc *resultForce, int id);


typedef struct {

  int type;
  ObjectInteract interact;
  double viscosity;
  void *data;

} OBJECT;




typedef struct resistanceline {
  
  WAMLoc e1;
  WAMLoc e2;
  double force;

} RESISTANCELINE;


typedef struct {
  double xPlaneNeg;
  double xPlanePos;
  double yPlaneNeg;
  double yPlanePos;
  double zPlaneNeg;
  double zPlanePos;
} BLOCK;

typedef struct {
  double xPlaneNeg;
  double xPlanePos;
  double yPlaneNeg;
  double yPlanePos;
  double zPlaneNeg;
  double zPlanePos;
} CONTAINER;




typedef struct {
  double normal[3];
  double distance;
  int WAMSideSign;
} PLANE;

typedef struct {

  WAMLoc center;
  double radius;

} SPHERE;
 
typedef struct {
  
  double F[3];
} FIELD;


int effectGandFComp(wam_struct *thisWAM, void *data);
int hapticObjectInteractor(WAMdat *wd, WAMLoc *tipforce, void *data);
int blockInteract(WAMLoc *curpos, WAMLoc *tipforce, int id);
int containerInteract(WAMLoc *curpos, WAMLoc *tipforce, int id);
void sphereInteract(WAMLoc *cursorpos, WAMLoc *tipforce, int id);
int resistanceLineInteract(WAMLoc *wd, WAMLoc *tipforce, int id);
void fieldInteract(WAMLoc *curpos, WAMLoc *tipforce, int id);

int planeInteract(WAMLoc *wd, WAMLoc *tipforce, int id);
void deleteClosePlane(WAMLoc *tipLoc);



#endif
/*======================================================================*
 *                                                                      *
 *          Copyright (c) 2003, 2004 Barrett Technology, Inc.           *
 *                        625 Mount Auburn St                           *
 *                    Cambridge, MA  02138,  USA                        *
 *                                                                      *
 *                        All rights reserved.                          *
 *                                                                      *
 *  ******************************************************************  *
 *                            DISCLAIMER                                *
 *                                                                      *
 *  This software and related documentation are provided to you on      *
 *  an as is basis and without warranty of any kind.  No warranties,    *
 *  express or implied, including, without limitation, any warranties   *
 *  of merchantability or fitness for a particular purpose are being    *
 *  provided by Barrett Technology, Inc.  In no event shall Barrett     *
 *  Technology, Inc. be liable for any lost development expenses, lost  *
 *  lost profits, or any incidental, special, or consequential damage.  *
 *======================================================================*/
