/*======================================================================*
 *  Module .............libbtwam
 *  File ...............newlibbt.h
 *  Author .............Sam Clanton
 *  Creation Date ......Q3 2004
 *  Addt'l Authors......Traveler Hauptman
 *                                                                      *
 *  ******************************************************************  *
 *                                                                      *
 *  NOTES:
 *
 *  REVISION HISTORY:
 *    Dec 17, 2004: TH
 *      Splitting libbt into libbtsystem & libbtwam. Adjust file names
 *      to support this. btwam -> btwamctl, btdriver -> btwam
 *                                                                      *
 *======================================================================*/
#ifndef __NEWLIBBT_H__
#define __NEWLIBBT_H__

#include "btwamctl.h"
#include "Kinematics.h"
#include "Dynamics.h"
#include "Haptics.h"



/* defined in Kinematics.h */

/* typedef struct { */

/*   double pos[3]; */
/*   double angle[3]; */

/* } WAMLoc; */


/* defined in Haptics.h */
/* typedef struct { */
  
/*   wam_struct *wamDriverData; //temporarily a pointer */
/*   WAMLoc wamTipLoc; */
/*   WAMKinematics wamKin; */
/*   WAMDynamics wamDyn; */


/* } WAMdat; */




/* Using jacobian transpose */
void StaticF2torque(WAMKinematics *wamKin, wam_vector *input_angles, double *fVec, wam_vector *torques);

void findJacobian(WAMKinematics *wamKin, wam_vector *input_angles, double *Jacobian);


void InitHapticWAM(WAMdat *wam);

//void InitWAMDynamics();

/* Kinematics Routines */

void InitWAMKinematics(WAMKinematics *wamKin, int dof);
void WAMFWKin(WAMKinematics *wamKin, wam_vector *input_angles, WAMLoc *outputLoc);
void FindTransformedPoint(WAMKinematics *wamKin,  wam_vector *inputAngles, WAMLoc *inputLoc, WAMLoc *outputLoc); /* finds point in base frame in tip frame */

/* Dynamics Routines */

void InitWAMDynamics(WAMDynamics *wam, int dof);
int getGcomp(wam_struct *);
void setGcomp(wam_struct *, int onoff);
void toggleGcomp(wam_struct *);

/* Haptics Routines */

void InitHaptics();

int registerForceEffector(ForceEffector thisForceEffector);
void deregisterForceEffector(int index);

void removeHapticObject(int index);
int addHapticObject(OBJECT *object);

void createBlock(OBJECT *inputObj, BLOCK *inputBlock, double *c1, double *c2, double viscosity); /* two opposite corners */
void createContainer(OBJECT *inputObj, CONTAINER *inputBlock, double *c1, double *c2); /* two opposite corners */
void createSphere(OBJECT *inputObj, SPHERE *inputSphere, double *c1, double *c2); 
/* center, radius */
void createResistanceLine(OBJECT *inputObj, RESISTANCELINE *inputLine, WAMLoc *e1, WAMLoc *e2, double force);
int createPlane(OBJECT *inputObj, PLANE *inputPlane, double *pt1, double *pt2, double *pt3);
void createField(OBJECT *inputObj, FIELD *inputField, double *pt1, double *pt2);

/* find distance from WAM to plane */
double distanceFromPlane(WAMLoc *loc, PLANE *plane);

#endif
/*======================================================================*
 *                                                                      *
 *          Copyright (c) 2003, 2004 Barrett Technology, Inc.           *
 *                        625 Mount Auburn St                           *
 *                    Cambridge, MA  02138,  USA                        *
 *                                                                      *
 *                        All rights reserved.                          *
 *                                                                      *
 *  ******************************************************************  *
 *                            DISCLAIMER                                *
 *                                                                      *
 *  This software and related documentation are provided to you on      *
 *  an as is basis and without warranty of any kind.  No warranties,    *
 *  express or implied, including, without limitation, any warranties   *
 *  of merchantability or fitness for a particular purpose are being    *
 *  provided by Barrett Technology, Inc.  In no event shall Barrett     *
 *  Technology, Inc. be liable for any lost development expenses, lost  *
 *  lost profits, or any incidental, special, or consequential damage.  *
 *======================================================================*/
